<?php

namespace Drupal\mymodule_migration;

use Drupal\Core\File\FileSystemInterface;
use Drupal\migrate\MigrateException;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

/**
 * The migration XML fetcher.
 */
class MyModuleXmlData {

  /**
   * Base folder for migration XML files.
   */
  const XML_DIRECTORY = '/tmp/migration_data/';

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * Static cache containing categories keyed by external ID.
   *
   * @var array
   */
  protected $categoriesById;

  /**
   * Static cache containing components keyed by external ID.
   *
   * @var array
   */
  protected $componentsById;

  /**
   * Static cache containing components keyed by article ID.
   *
   * @var array
   */
  protected $componentsByArticleId;

  /**
   * Static cache containing articles keyed by external ID.
   *
   * @var array
   */
  protected $articlesById;

  /**
   * Static cache containing images keyed by URL.
   *
   * @var array
   */
  protected $images;

  /**
   * Static cache containing users keyed by ID.
   *
   * @var array
   */
  protected $userIds;

  /**
   * Construct a MyModuleXmlData object.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   */
  public function __construct(FileSystemInterface $file_system, Client $http_client) {
    $this->fileSystem = $file_system;
    $this->httpClient = $http_client;
  }

  /**
   * Fetch all categories from the XML export.
   *
   * @return array
   *   An array containing all categories keyed by external ID.
   */
  public function getCategories() {
    // If we already have a statically cached version for our result, return
    // that!
    if (!empty($this->categoriesById)) {
      return $this->categoriesById;
    }

    $categories_xml = simplexml_load_file($this::XML_DIRECTORY . 'categories.xml', 'SimpleXMLElement', LIBXML_NOCDATA);

    $this->categoriesById = [];
    foreach ($categories_xml as $category) {
      $id = $category->categoryId->__toString();
      $this->categoriesById[$id] = [
        'id' => $id,
        'name' => trim($category->categoryName->__toString()),
        'slug' => $category->categorySlug->__toString(),
      ];
    }

    return $this->categoriesById;
  }

  /**
   * Fetch all components from the XML export.
   *
   * @return array
   *   An array containing all categories keyed by external ID.
   */
  public function getComponents() {
    // If we already have a statically cached version for our result, return
    // that!
    if (!empty($this->componentsById)) {
      return $this->componentsById;
    }

    $component_categories_xml = simplexml_load_file($this::XML_DIRECTORY . 'components.xml', 'SimpleXMLElement', LIBXML_NOCDATA);

    $this->componentsById = [];
    foreach ($component_categories_xml as $component_category) {
      $id = $component_category->componentcategoryId->__toString();

      $this->componentsById[$id] = [
        'id' => $id,
        'category' => $component_category->categoryId->__toString(),
        'body' => $component_category->componentBody->__toString(),
      ];
    }

    return $this->componentsById;
  }

  /**
   * Fetch all components from the XML export.
   *
   * @return array
   *   A nested array containing all article components keyed by article ID.
   */
  protected function getComponentsByArticleIdList() {
    // If we already have a statically cached version for our result, return
    // that!
    if (!empty($this->componentsByArticleId)) {
      return $this->componentsByArticleId;
    }

    $article_components_xml = simplexml_load_file($this::XML_DIRECTORY . 'components.xml', 'SimpleXMLElement', LIBXML_NOCDATA);
    $this->componentsByArticleId = [];
    foreach ($article_components_xml as $component) {
      $this->componentsByArticleId[$component->articleId->__toString()][] = $component->componentId->__toString();
    }
    return $this->componentsByArticleId;
  }

  /**
   * Fetch all article components from the XML export.
   *
   * @param int $article_id
   *   The article ID to fetch the components for.
   *
   * @return array
   *   An array containing all article components for the article ID.
   */
  protected function getComponentsByArticleId($article_id) {
    $article_component_list = $this->getComponentsByArticleIdList();
    return isset($article_component_list[$article_id]) ? $article_component_list[$article_id] : [];
  }

  /**
   * Fetch all articles from the XML export.
   *
   * @return array
   *   An array containing all articles keyed by external ID.
   */
  public function getArticles() {
    // If we already have a statically cached version for our result, return
    // that!
    if (!empty($this->articlesById)) {
      return $this->articlesById;
    }

    $articles_xml = simplexml_load_file($this::XML_DIRECTORY . 'articles.xml', 'SimpleXMLElement', LIBXML_NOCDATA);

    $this->articlesById = [];
    foreach ($articles_xml as $article) {
      $id = $article->articleId->__toString();

      $article_components = $this->getComponentsByArticleId($id);

      $this->articlesById[$id] = [
        'id' => $id,
        'name' => trim($article->articleName->__toString()),
        'summary' => $article->articleSummary->__toString(),
        'description' => $article->articleDescription->__toString(),
        'category' => $article->articleCategory->__toString(),
        'image' => $article->images->main->__toString(),
        'paragraph_components' => $article_components,
        'slug' => '/article/' . $article->articleSlug->__toString(),
        'moderation_state' => $article->articlePublished->__toString() ? 'published' : 'draft',
        'status' => $article->articlePublished->__toString() ? 1 : 0,
        'created' => strtotime($article->articleCreatedAt->__toString()),
        'changed' => strtotime($article->articleModifiedAt->__toString()),
      ];
    }

    return $this->articlesById;
  }

  /**
   * Fetch all images from the XML exports.
   *
   * The images are downloaded as well. We only download images if they do not
   * exist yet (this will save time when doing updates).
   *
   * @return array
   *   An array containing all images keyed by URL.
   */
  public function getImages() {
    // If we already have a statically cached version for our result, return
    // that!
    if (!empty($this->images)) {
      return $this->images;
    }

    $this->images = [];

    // Article images.
    $articles = $this->getArticles();
    foreach ($articles as $article) {
      if (!empty($article['image']) && $file_data = $this->downloadFile($article['image'], $article['created'])) {
        $this->images[$article['image']] = $file_data;
        $this->images[$article['image']]['name'] = $article['name'];
      }
    }

    return $this->images;
  }

  /**
   * Download a file URL if it doesn't exist yet.
   *
   * @param string $url
   *   The URL of the file to download.
   * @param int $created_timestamp
   *   The UNIX timestamp to use as the created date.
   *
   * @return string
   *   The URI of the downloaded file.
   */
  protected function downloadFile($url, $created_timestamp) {
    $dir = 'public://' . date('Y', $created_timestamp) . '-' . date('m', $created_timestamp);
    if (!file_prepare_directory($dir, FILE_CREATE_DIRECTORY)) {
      \Drupal::logger('file')->error('The directory %dir could not be created. Please check the file permissions.', ['%dir' => $dir]);
      return FALSE;
    }

    $filename = $this->fileSystem->basename($url);
    $path = $dir . '/' . $filename;
    if (!file_exists($path)) {
      try {
        $data = (string) $this->httpClient->get($url)->getBody();
        $downloaded = file_unmanaged_save_data($data, $path, FILE_EXISTS_REPLACE);

        if (!$downloaded) {
          \Drupal::logger('file')->error('The file %url could not be downloaded. Please check the URL.', ['%url' => $url]);
          return FALSE;
        }
      }
      catch (RequestException $exception) {
        return FALSE;
      }
    }

    return [
      'id' => $url,
      'filename' => $filename,
      'path' => $path,
    ];
  }

  /**
   * Fetch all users from the XML export.
   *
   * @return array
   *   An array containing all users keyed by external ID.
   */
  public function getUsers() {

    $users_xml = simplexml_load_file($this::XML_DIRECTORY . 'users.xml', 'SimpleXMLElement', LIBXML_NOCDATA);

    $users = [];
    $this->userIds = [];
    foreach ($users_xml as $user) {
      $id = $user->user_id->__toString();
      $active = strtolower($user->user_status->__toString()) === 'online';

      // Add static cache record.
      $this->userIds[$id] = $id;

      // Add import data.
      $users[$id] = [
        'id' => $id,
        'name' => $user->user_username->__toString(),
        'email' => $user->user_email->__toString(),
        'firstname' => $user->user_first_name->__toString(),
        'lastname' => $user->user_last_name->__toString(),
        'status' => $active ? 1 : 0,
        'created' => strtotime($user->user_created_at->__toString()),
        'changed' => strtotime($user->user_modified_at->__toString()),
        'last_login' => strtotime($user->user_last_login_at->__toString()),
      ];
    }

    return $users;
  }

}
